/*****************************************/
// dragon for Titchaino and ikkeiplexing shield
// by ikkei 2014.07.09
// 
#include <arduino.h>
#include <ikkeiplexing.h>

#include "dragon.h"

static ikkeiplexing iLedSign;

#define XLED 8
#define YLED 16
#define XMAX 7
#define YMAX 15
#define SENS 1  // Sensitivity
#define MAG 1024  // Magnification
#define REF1 3  // Reflectance is REF1/REF2
#define REF2 4

static uint8_t plane[XLED][YLED] ={ 0 };
static uint8_t backcolor = 0;
static uint8_t ballcolor = 1;
static int16_t vel_x[10] = {0};  // velocity
static int16_t vel_y[10] = {0};
static int16_t pos_x[10] = {0};  // position
static int16_t pos_y[10] = {0};

void dragon_init(){
  iLedSign.Clear();
  for (byte x=0; x<XLED; x++){
    for (byte y=0; y<YLED; y++){
        plane[x][y] = backcolor;
    }
  }
  for (byte i=0; i<10; i++){
    pos_x[i] = MAG * XLED /2;
  }
}

void dragon(){
  const uint8_t X_NUM = XLED;
  const uint8_t Y_NUM = YLED;
  const uint8_t X_MAX = XMAX;
  const uint8_t Y_MAX = YMAX;
  //static int x1 = 1;  // display next position
  //static int y1 = 1;
  //static int x0 = 1;  // display last position
  //static int y0 = 1;
  static int x1[10] = {0};
  static int y1[10] = {0};
  static int x0[10] = {0};
  static int y0[10] = {0};
  
  static int acc_x;  // acceleration
  static int acc_y;
  //static int vel_x;  // velocity
  //static int vel_y;

  
  static int pol_x; // Polarity x
  static int pol_y; // Polarity y
  static byte time[10] = {10, 20, 30, 40, 50, 60, 70, 80, 90, 100};
  byte i;
  static unsigned long fadeout[20] = { //0.5
  //0,0,0,0,0,0,0,0,0,0,
  0,0,0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 1, 2, 3, 7, 15, 31, 63, 127,
  };

  //fill_color(backcolor);
  //attenuate( backcolor, 150); // 200/255

  acc_x = 0;
  acc_y = -40;
  
  for(i=0; i<10; i++){
    time[i]--;
    if (time[i] == 0){
      time[i] = random(30,37);
      //time[i] = 40;
      pos_y[i] = random(0,2000);
      pos_x[i] = MAG * XLED /2;
      vel_y[i] = random(1050,1100);
      //vel_y[i] = 1100;
      vel_x[i] = random(-90,90);
    }
  
    // integrating acceleration makes velocity
    vel_x[i] += acc_x;
    vel_y[i] += acc_y;

    // integrating velocity makes position
    pos_x[i] += vel_x[i];
    pos_y[i] += vel_y[i];
  
    // reflection ( reverse velocity )
    // 0 < x1 < X_MAX
    if ( pos_x[i] >= MAG * X_MAX){
      pos_x[i] = MAG * X_MAX;
      vel_x[i] = -( vel_x[i] * REF1 ) / REF2;
    }else if ( pos_x[i] < 0 ){
      pos_x[i] = 0;
      vel_x[i] = -( vel_x[i] * REF1 ) / REF2;
    }
    // 0 < y1 < Y_MAX
    if ( pos_y[i] >= MAG * Y_MAX){
      pos_y[i] = MAG * Y_MAX;
      vel_y[i] = -( vel_y[i] * REF1 ) / REF2;
    }else if ( pos_y[i] < 0 ){
      pos_y[i] = 0;
      vel_y[i] = -( vel_y[i] * REF1 ) / REF2;
    }
  
//  x1 = acc_x + 8;    // test acceleration value
//  y1 = acc_y + 4;
//  x1 = vel_x / 16 + 8;  // test velocity value
//  y1 = vel_y / 16 + 4;
    x1[i] = pos_x[i] / MAG;
    y1[i] = pos_y[i] / MAG;
  
    // display limitter
    if ( x1[i] > X_MAX ) {
      x1[i] = X_MAX;
    //}else if ( x1 < 1 ){
    //  x1 = 1;
    }
    if ( y1[i] > Y_MAX ) {
      y1[i] = Y_MAX;
    //}else if (y1 < 1){
    //  y1 = 1;
    }

    // turn off the last position
    plane[ x0[i] ][ y0[i] ] = backcolor;
    
    // turn on the next position
//    if (time[i] > 20){
//      ballcolor = WHITE;
//    } else {
//      ballcolor = calcColor(ballcolor, backcolor, fadeout[time[i]]);
//    }
    if (x1[i] < XLED && y1[i] < YLED){
      plane[ x1[i] ][ y1[i] ] = ballcolor;
    }
    
    // store last position
    x0[i] = x1[i];
    y0[i] = y1[i];

  }    
  for (byte x=0; x<XLED; x++){
    for (byte y=0; y<YLED; y++){
        iLedSign.Set(y, x, plane[x][y]);
    }
  }
   
}

